import os
from pathlib import Path

import aiofiles

from autopack.filesystem_emulation.file_manager import FileManager
from autopack.pack_config import PackConfig


class FileSystemManager(FileManager):
    """
    This class provides unrestricted file operations on the local file system.
    """

    def __init__(self, config: PackConfig = PackConfig.global_config()):
        super().__init__(config)

    def read_file(self, file_path: str) -> str:
        """Reads a file from the local file system.

        Args:
            file_path (str): The absolute path to the file to be read.

        Returns:
            str: The content of the file. If the file does not exist, returns an error message.
        """
        absolute_path = Path(file_path)
        if absolute_path.exists():
            with open(absolute_path, "r") as file:
                return file.read()
        else:
            return "Error: File not found"

    def write_file(self, file_path: str, content: str) -> str:
        """Writes to a file on the local file system.

        Args:
            file_path (str): The absolute path to the file to be written to.
            content (str): The content to be written to the file.

        Returns:
            str: A success message indicating the file was written.
        """
        absolute_path = Path(file_path)
        absolute_path.parent.mkdir(parents=True, exist_ok=True)

        with open(absolute_path, "w") as file:
            file.write(content)

        return f"Successfully wrote {len(content.encode('utf-8'))} bytes to {file_path}"

    async def awrite_file(self, file_path: str, content: str) -> str:
        """Writes to a file on the local file system.

        Args:
            file_path (str): The absolute path to the file to be written to.
            content (str): The content to be written to the file.

        Returns:
            str: A success message indicating the file was written.
        """
        absolute_path = Path(file_path)
        absolute_path.parent.mkdir(parents=True, exist_ok=True)

        async with aiofiles.open(absolute_path, "w") as file:
            await file.write(content)

        return f"Successfully wrote {len(content.encode('utf-8'))} bytes to {file_path}"

    def delete_file(self, file_path: str) -> str:
        """Deletes a file from the local file system.

        Args:
            file_path (str): The absolute path to the file to be deleted.

        Returns:
            str: A success message indicating the file was deleted. If the file does not exist, returns an error message.
        """
        absolute_path = Path(file_path)
        if absolute_path.exists():
            os.remove(absolute_path)
            return f"Successfully deleted file {file_path}."
        else:
            return f"Error: File not found '{file_path}'"

    def list_files(self, dir_path: str) -> str:
        """Lists all files in the specified directory on the local file system.

        Args:
            dir_path (str): The absolute path to the directory to list files from.

        Returns:
            str: A list of all files in the directory. If the directory does not exist, returns an error message.
        """
        absolute_dir_path = Path(dir_path)
        if absolute_dir_path.exists() and absolute_dir_path.is_dir():
            files_in_dir = absolute_dir_path.glob("*")
            return "\n".join(str(file) for file in files_in_dir if file not in self.IGNORE_FILES)
        else:
            return f"Error: No such directory {dir_path}."



from enum import Enum
from typing import TYPE_CHECKING, ClassVar, Optional

from pydantic import Field, BaseSettings

from autopack.filesystem_emulation.file_manager import FileManager

if TYPE_CHECKING:
    from autopack import Pack


class InstallerStyle(str, Enum):
    # Packs and dependencies are automatically installed on use
    automatic = "automatic"
    # Packs, but not dependencies, are installed automatically on use
    semiautomatic = "semiautomatic"
    # Packs installed either through CLI or through your own code calling `install_pack`
    manual = "manual"


class PackConfig(BaseSettings):
    """
    Class for defining the configuration of AutoPack. This will either be set by:
    - A custom instantiated class or subclass, and then passed to the various autopack functions
    - A custom instantiated class or subclass, and then set as the global config
    - An automatically-configured global config based on environment variables and hard-coded defaults

    For each of these configuration settings there is a corresponding environment variable prefixed with `AUTOPACK_`,
    e.g.: AUTOPACK_FILESYSTEM_TYPE, AUTPACK_WORKSPACE_PATH
    """

    _global_config: ClassVar["PackConfig"] = None
    filesystem_manager: Optional[FileManager] = None

    class Config:
        env_prefix = "AUTOPACK_"

    workspace_path: str = Field(
        description="The directory for artifact storage and storage of working files.", default="workspace"
    )
    installer_style: InstallerStyle = Field(
        description="Style and permissiveness of Pack installation", default=InstallerStyle.automatic
    )
    restrict_code_execution: bool = Field(
        description="If True will signal to Packs that they should not execute code", default=False
    )
    # Not implemented yet
    local_packs: list[type["Pack"]] = Field(
        description="A list of local Pack classes that you wish to be included in the selection process",
        default_factory=list,
    )

    @classmethod
    def set_global_config(cls, config_obj: "PackConfig" = None) -> "PackConfig":
        """
        Set the default global config to either a PackConfig object of your choice, or `None` to use the default.
        """
        cls._global_config = config_obj or cls()
        return cls._global_config

    @classmethod
    def global_config(cls) -> "PackConfig":
        return cls._global_config or cls.set_global_config()

    def init_filesystem_manager(self, file_manager: type["FileManager"] = None):
        if file_manager:
            # Override previous / set new manager
            self.filesystem_manager = file_manager(self)
        elif not self.filesystem_manager:
            # Default
            from autopack.filesystem_emulation.workspace_file_manager import WorkspaceFileManager

            self.filesystem_manager = WorkspaceFileManager(self)

    @classmethod
    def create(cls, **kwargs):
        instance = cls(**kwargs)
        instance.init_filesystem_manager()
        return instance

    @property
    def automatically_install_dependencies(self) -> bool:
        return self.installer_style == InstallerStyle.automatic



from autopack.filesystem_emulation.file_manager import FileManager
from autopack.pack_config import PackConfig


class RAMFileManager(FileManager):
    """
    This class emulates a filesystem in RAM, storing files in a dictionary where keys are file paths and values are
    file content. Recommended for sandboxing or for testing.
    """

    def __init__(self, config: PackConfig = PackConfig.global_config()):
        super().__init__(config)
        self.files = {}

    def read_file(self, file_path: str) -> str:
        """Reads a file from the virtual file system in RAM.

        Args:
            file_path (str): The path to the file to be read.

        Returns:
            str: The content of the file. If the file does not exist, returns an error message.
        """
        if file_path in self.files:
            return self.files[file_path]
        else:
            return "Error: File not found"

    def write_file(self, file_path: str, content: str) -> str:
        """Writes to a file in the virtual file system in RAM.

        Args:
            file_path (str): The path to the file to be written to.
            content (str): The content to be written to the file.

        Returns:
            str: A success message indicating the file was written.
        """
        self.files[file_path] = content
        return f"Successfully wrote {len(content.encode('utf-8'))} bytes to {file_path}"

    async def awrite_file(self, file_path: str, content: str) -> str:
        return self.write_file(file_path, content)

    def delete_file(self, file_path: str) -> str:
        """Deletes a file from the virtual file system in RAM.

        Args:
            file_path (str): The path to the file to be deleted.

        Returns:
            str: A success message indicating the file was deleted. If the file does not exist, returns an error message.
        """
        if file_path in self.files:
            del self.files[file_path]
            return f"Successfully deleted file {file_path}."
        else:
            return f"Error: File not found '{file_path}'"

    def list_files(self, dir_path: str) -> str:
        """Lists all files in the specified directory in the virtual file system in RAM.

        Args:
            dir_path (str): The path to the directory to list files from.

        Returns:
            str: A list of all files in the directory. If the directory does not exist, returns an error message.
        """
        # For simplicity, let's assume that all keys in `self.files` are file paths, and to list files in a directory,
        # we just need to find all keys that start with `dir_path`.
        files_in_dir = [
            file_path
            for file_path in self.files.keys()
            if file_path.startswith(dir_path) and file_path not in self.IGNORE_FILES
        ]
        if files_in_dir:
            return "\n".join(files_in_dir)
        else:
            return f"Error: No such directory {dir_path}."



from dataclasses import dataclass, field

from dataclasses_json import dataclass_json


@dataclass_json
@dataclass
class PackResponse:
    """Class to store metadata about a (possibly uninstalled) pack"""

    pack_id: str
    package_path: str
    class_name: str
    repo_url: str
    name: str
    description: str
    dependencies: list[str] = field(default_factory=list)
    run_args: dict[str, dict[str, str]] = field(default_factory=list)
    categories: list[str] = field(default_factory=list)



from abc import ABC, abstractmethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from autopack.pack_config import PackConfig


class FileManager(ABC):
    # A few Packs will use poetry inside of the workspace, and the AI gets hella confused when these files are present.
    IGNORE_FILES = ["pyproject.toml", "poetry.lock"]

    def __init__(self, config: "PackConfig" = None):
        from autopack.pack_config import PackConfig

        self.config = config or PackConfig.global_config()

    @abstractmethod
    def read_file(self, file_path: str) -> str:
        pass

    @abstractmethod
    def write_file(self, file_path: str, content: str) -> str:
        pass

    @abstractmethod
    def delete_file(self, file_path: str) -> str:
        pass

    @abstractmethod
    def list_files(self, dir_path: str) -> str:
        pass
